@@ -23,6 +23,190 @@
 # (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 # SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
+#' @name exportDDI
+#' @title
+#' Export to a DDI metadata file
+#'
+#' @description
+#' Create a DDI Codebook version 2.5, XML file structure.
+#'
+#' @details
+#' #' The information object is essentially a list having two main list components:
+#'
+#' - \bold{\code{fileDscr}}, if the data is provided in a subcomponent named
+#' \bold{\code{datafile}}
+#'
+#' - \bold{\code{dataDscr}}, having as many components as the number of variables
+#' in the (meta)data. For each variable, there should a mandatory subcomponent
+#' called \bold{\code{label}} (that contains the variable's label) and, if the
+#' variable is of a categorical type, another subcomponent called
+#' \bold{\code{values}}.
+#'
+#' Additional informations about the variables can be specified as further
+#' subcomponents, combining DDI specific data but also other information that might
+#' not be covered by DDI:
+#'
+#' - \bold{\code{measurement}} is the equivalent of the specific DDI attribute
+#' \bold{\code{nature}} of the \bold{\code{var}} element, and it accepts these
+#' values: \code{"nominal"}, \code{"ordinal"}, \code{"interval"}, \code{"ratio"},
+#' \code{"percent"}, and \code{"other"}.
+#'
+#' - \bold{\code{type}} is useful for multiple reasons. A first one, if the
+#' variable is numerical, is to differentiate between \code{discrete} and
+#' \code{contin} values of the attribute \bold{\code{intrvl}} from the same DDI
+#' element \bold{\code{var}}. Another reason is to help identifying pure string
+#' variables (containing text), when the subcomponent \bold{\code{type}} is equal
+#' to \code{"char"}. It is also used for the subelement \bold{\code{varFormat}} of
+#' the element \bold{\code{var}}. Finally, another reason is to differentiate
+#' between pure categorical (\code{"cat"}) and pure numerical (\code{"num"})
+#' variables, as well as mixed ones, among which \code{"numcat"} referring to a
+#' numerical variable with very few values (such as the number of children), for
+#' which it is possible to also produce a table of frequencies along the numerical
+#' summaries. There are also categorical variables that can be interpreted as
+#' numeric (\code{"catnum"}), such as a Likert type response scale with 7 values,
+#' where numerical summaries are also routinely performed along with the usual
+#' table of frequencies.
+#'
+#' - \bold{\code{missing}} is an important subcomponent, indicating which of the
+#' values in the variable are going to be treated as missing values, and it is
+#' going to be exported as the attribute \code{missing} of the DDI subelement
+#' \bold{\code{catgry}}.
+#'
+#' There are many more possible attributes and DDI elements to be added in the
+#' information object, future versions of this function will likely expand.
+#'
+#' For the moment, only DDI codebook version 2.5 is exported, and DDI Lifecycle is
+#' planned for future releases.
+#'
+#' Argument \bold{\code{xmlang}} expects a two letter ISO country coding, for
+#' instance \code{"en"} to indicate English, or \code{"ro"} to indicate Romanian etc.
+#'
+#' If the document is monolang, this argument is placed a single time for the
+#' entire document, in the attributes of the \code{codeBook} element. For
+#' multilingual documents, it is placed in the attributes of various other
+#' (sub)elements, for instance \code{abstract} as an obvious one, or the study
+#' title, name of the distributing institution, variable labels etc.
+#'
+#' The argument \bold{\code{OS}} can be either:\cr
+#' \code{"windows"} (default), or \code{"Windows"}, \code{"Win"}, \code{"win"},\cr
+#' \code{"MacOS"}, \code{"Darwin"}, \code{"Apple"}, \code{"Mac"}, \code{"mac"},\cr
+#' \code{"Linux"}, \code{"linux"}.
+#'
+#' The end of line separator changes only when the target OS is different from the
+#' running OS.
+#'
+#' The argument \bold{\code{indent}} controls how many spaces will be used in the
+#' XML file, to indent the different subelements.
+#'
+#' A small number of required DDI specific elements and attributes have generic
+#' default values but they may be specified using the three dots \bold{\code{...}}
+#' argument. For the current version, these are: \code{IDNo}, \code{titl},
+#' \code{agency}, \code{URI} (for the \code{holdings} element), \code{distrbtr},
+#' \code{abstract} and \code{level} (for the \code{otherMat} element).
+#'
+#' @return
+#' An XML file containing a DDI version 2.5 metadata.
+#'
+#' @seealso
+#' \url{https://ddialliance.org/Specification/DDI-Codebook/2.5/XMLSchema/field_level_documentation.html}
+#'
+#' @examples
+#' codeBook <- list(dataDscr = list(
+#' ID = list(
+#'     label = "Questionnaire ID",
+#'     type = "num",
+#'     measurement = "interval"
+#' ),
+#' V1 = list(
+#'     label = "Label for the first variable",
+#'     labels = c(
+#'         "No"             =  0,
+#'         "Yes"            =  1,
+#'         "Not applicable" = -97,
+#'         "Not answered"   = -99),
+#'     na_values = c(-99, -97),
+#'     type = "cat",
+#'     measurement = "nominal"
+#' ),
+#' V2 = list(
+#'     label = "Label for the second variable",
+#'     labels = c(
+#'         "Very little"    =  1,
+#'         "Little"         =  2,
+#'         "So, so"         =  3,
+#'         "Much"           =  4,
+#'         "Very much"      =  5,
+#'         "Don't know"     = -98),
+#'     na_values = c(-98),
+#'     type = "cat",
+#'     measurement = "ordinal"
+#' ),
+#' V3 = list(
+#'     label = "Label for the third variable",
+#'     labels = c(
+#'         "First answer"   = "A",
+#'         "Second answer"  = "B",
+#'         "Don't know"     = -98),
+#'     na_values = c(-98),
+#'     type = "cat",
+#'     measurement = "nominal"
+#' ),
+#' V4 = list(
+#'     label = "Number of children",
+#'     labels = c(
+#'         "Don't know"     = -98,
+#'         "Not answered"   = -99),
+#'     na_values = c(-99, -98),
+#'     type = "numcat",
+#'     measurement = "ratio"
+#' ),
+#' V5 = list(
+#'     label = "Political party reference",
+#'     type = "char",
+#'     txt = "When the respondent indicated his political party reference, his/her
+#'         open response was recoded on a scale of 1-99 with parties with a
+#'         left-wing orientation coded on the low end of the scale and parties with
+#'         a right-wing orientation coded on the high end of the scale. Categories
+#'         90-99 were reserved miscellaneous responses."
+#' )))
+#'
+#' \dontrun{
+#' exportDDI(codeBook, file = "codebook.xml")
+#'
+#' # using a namespace
+#' exportDDI(codeBook, file = "codebook.xml", xmlns = "ddi")
+#' }
+#'
+#' @author Adrian Dusa
+#'
+#' @param codebook
+#' A list object containing the metadata, or a path to a directory
+#' where these objects are located, for batch processing
+#'
+#' @param file
+#' either a character string naming a file or a connection open for
+#' writing. "" indicates output to the console
+#'
+#' @param embed
+#' Logical, embed the CSV datafile in the XML file, if present
+#'
+#' @param OS
+#' The target operating system, for the eol - end of line character(s)
+#'
+#' @param indent Indent width, in number of spaces
+#'
+#' @param monolang Logical, monolang or multilingual document
+#'
+#' @param xmlang ISO two letter code for the language used in the DDI elements
+#'
+#' @param xmlns
+#' Character, namespace for the XML file (ignored if already present
+#' in the codebook object)
+#'
+#' @param ... Other arguments, mainly for internal use
+#'
+#' @export
+
 `exportDDI` <- function(
     codebook, file = "", embed = TRUE, OS = "", indent = 4,
     monolang = FALSE, xmlang = "en", xmlns = "", ...
@@ -34,7 +218,7 @@
     # schema <- read_xml("path/to/ddi_2_5_1/schemas/codebook.xsd")
     # doc <- read_xml("path/to/filetovalidate.xml")
     # xml_validate(doc, schema)
-    
+
     `check_arg` <- function(x, vx, type = "c") {
         valid <- is.atomic(vx) && length(vx) == 1
         if (valid & type == "c") { # character
@@ -78,7 +262,7 @@
         # toreturn <- unique(toreturn)
         # if (length(toreturn) == x) return(toreturn)
     }
-    
+
     `repeatSpace` <- function(times) {
         paste(rep(" ", times*indent), collapse = "")
     }
@@ -102,7 +286,7 @@
     level <- check_dots("level", "0")
 
     check_arg("xmlang", xmlang)
-    
+
     xmlang <- paste0(
         ifelse(isTRUE(monolang), "", " "),
         "xml:lang=\"", xmlang, "\""
@@ -130,7 +314,7 @@
             sep = ""
         ))
     }
-    
+
     if (OS == "") {
         OS <- Sys.info()[["sysname"]]
     }
@@ -138,16 +322,17 @@
     enter <- getEnter(OS = OS)
 
     data <- codebook[["fileDscr"]][["datafile"]]
-    dataDscr  <- codebook[["dataDscr"]]
+    stdyDscr <- codebook[["stdyDscr"]]
+    dataDscr <- codebook[["dataDscr"]]
     pN <- logical(length(dataDscr))
-    
+
     # uuid for all variables
     uuid <- generateUUID(length(dataDscr))
-    
+
     prodate <- as.character(Sys.time())
     version <- as.character(packageVersion("DDIwR"))
     varnames <- names(dataDscr)
-    
+
     if (!identical(file, "")) {
         sink(file)
         on.exit(sink())
@@ -170,9 +355,9 @@
         # enter,
         ifelse(isTRUE(monolang), paste0(xmlang, enter), ""),
         "xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\"",
-        enter, 
+        enter,
         "xmlns:xsd=\"http://www.w3.org/2001/XMLSchema\"",
-        enter, 
+        enter,
         "xsi:schemaLocation=\"",
         "ddi:codebook:2_5 https://ddialliance.org/Specification/DDI-Codebook/2.5/XMLSchema/codebook.xsd\"",
         enter,
@@ -186,7 +371,7 @@
     if (isTRUE(monolang)) {
         xmlang <- ""
     }
-    
+
     # Document description
     cat(paste(s1, "<", ns, "docDscr>", enter, sep = ""))
     cat(paste(s2, "<", ns, "citation>", enter, sep = ""))
@@ -219,47 +404,73 @@
     cat(paste(s1, "</", ns, "docDscr>", enter, sep = ""))
 
     # Study description
-    cat(paste(s1, "<", ns, "stdyDscr>", enter, sep = ""))
-    cat(paste(s2, "<", ns, "citation>", enter, sep = ""))
+    if (is.null(stdyDscr)) {
+        cat(paste(s1, "<", ns, "stdyDscr>", enter, sep = ""))
+        cat(paste(s2, "<", ns, "citation>", enter, sep = ""))
 
-    cat(paste(s3, "<", ns, "titlStmt>", enter, sep = ""))
-    cat(paste(
-        s4, "<", ns, "titl", xmlang, ">", titl, "</", ns, "titl>",
-        enter,
-        sep = ""
-    ))
-    cat(paste(
-        s4, "<", ns, "IDNo agency=\"", agency,"\">", IDNo, "</", ns, "IDNo>",
-        enter,
-        sep = ""
-    ))
-    cat(paste(s3, "</", ns, "titlStmt>", enter, sep = ""))
-    
-    cat(paste(s3, "<", ns, "distStmt>", enter, sep = ""))
-    cat(paste(
-        s4, "<", ns, "distrbtr", xmlang, ">", distrbtr,
-        "</", ns, "distrbtr>",
-        enter, sep = ""
-    ))
-    cat(paste(s3, "</", ns, "distStmt>", enter, sep = ""))
+        cat(paste(s3, "<", ns, "titlStmt>", enter, sep = ""))
+        cat(paste(
+            s4, "<", ns, "titl", xmlang, ">", titl, "</", ns, "titl>",
+            enter,
+            sep = ""
+        ))
+        cat(paste(
+            s4, "<", ns, "IDNo agency=\"", agency,"\">", IDNo, "</", ns, "IDNo>",
+            enter,
+            sep = ""
+        ))
+        cat(paste(s3, "</", ns, "titlStmt>", enter, sep = ""))
 
-    cat(paste(
-        s3, "<", ns, "holdings URI=\"", URI,
-        "\">Description of the study holdings</", ns, "holdings>",
-        enter, sep = ""
-    ))
+        cat(paste(s3, "<", ns, "distStmt>", enter, sep = ""))
+        cat(paste(
+            s4, "<", ns, "distrbtr", xmlang, ">", distrbtr,
+            "</", ns, "distrbtr>",
+            enter, sep = ""
+        ))
+        cat(paste(s3, "</", ns, "distStmt>", enter, sep = ""))
 
-    cat(paste(s2, "</", ns, "citation>", enter, sep = ""))
-    cat(paste(s2, "<", ns, "stdyInfo>", enter, sep = ""))
-    cat(paste(
-        s3, "<", ns, "abstract", xmlang, ">", abstract, "</", ns, "abstract>",
-        enter, sep = ""
-    ))
+        cat(paste(
+            s3, "<", ns, "holdings URI=\"", URI,
+            "\">Description of the study holdings</", ns, "holdings>",
+            enter, sep = ""
+        ))
 
-    cat(paste(s2, "</", ns, "stdyInfo>", enter, sep = ""))
+        cat(paste(s2, "</", ns, "citation>", enter, sep = ""))
+        cat(paste(s2, "<", ns, "stdyInfo>", enter, sep = ""))
+        cat(paste(
+            s3, "<", ns, "abstract", xmlang, ">", abstract, "</", ns, "abstract>",
+            enter, sep = ""
+        ))
+
+        cat(paste(s2, "</", ns, "stdyInfo>", enter, sep = ""))
+
+        cat(paste(s1, "</", ns, "stdyDscr>", enter, sep = ""))
+    }
+    else {
+
+        cat(paste(makeXML(stdyDscr, 1, indent, ns, enter), collapse = ""))
+
+        ## when no xml namespace is needed, this works:
+        # stdyDscr <- as_xml_document(list(stdyDscr = stdyDscr))
+
+        # addns <- function(x) {
+        #     if (!is.null(names(x))) {
+        #         names(x) <- paste0(ns, names(x))
+        #         if (is.list(x)) {
+        #             x <- lapply(x, function(x) {
+        #                 addns(x)
+        #             })
+        #         }
+        #     }
+        #     return(x)
+        # }
+
+        # stdyDscr <- lapply(list(stdyDscr), addns)
+        # names(stdyDscr) <- paste0(ns, "stdyDscr")
+        # stdyDscr <- as_xml_document(stdyDscr) # error can not find namespace
+        # cat(as.character(stdyDscr))
+    }
 
-    cat(paste(s1, "</", ns, "stdyDscr>", enter, sep = ""))
-    
     fileDscrUUID <- generateUUID(1)
     cat(paste(
         s1, "<", ns, "fileDscr ID=\"", fileDscrUUID, "\">",
@@ -355,12 +566,12 @@
                 "\""
             )
         }
-        
+
         nature <- ""
         if(any(grepl("measurement", names(dataDscr[[i]])))) {
             nature <- paste0(" nature=\"", dataDscr[[i]]$measurement, "\"")
         }
-                         
+
         cat(paste0(
             s2, "<", ns, "var ID=\"", uuid[i], "\"",
             " name=\"", varnames[i], "\"",
@@ -368,7 +579,7 @@
             dcml, nature, ">",
             enter
         ))
-        
+
         if (!is.null(dataDscr[[i]][["label"]])) {
             if (!is.na(dataDscr[[i]][["label"]])) {
                 cat(paste(
@@ -382,7 +593,7 @@
                 ))
             }
         }
-        
+
         na_values <- NULL
         if (is.element("na_values", names(dataDscr[[i]]))) {
             na_values <- dataDscr[[i]]$na_values
@@ -431,7 +642,7 @@
                     sep = ""
                 ))
             }
-                
+
             cat(paste(s3, "</", ns, "invalrng>", enter, sep = ""))
         }
 
@@ -449,13 +660,13 @@
                     lbls[m] <- paste(strlab, collapse = "")
                 }
             }
-            
+
             lbls <- setNames(admisc::trimstr(lbls), nms)
             # names(lbls) <- nms
         }
 
         type <- dataDscr[[i]]$type
-        
+
         # even if the data is not present, pN is FALSE for all variables
         if (pN[i]) {
             vals <- aN[[names(dataDscr)[i]]]
@@ -474,12 +685,12 @@
             # (not just categorical using numbers)
             # if it has at least four(?) values different from the labels
             printnum <- length(setdiff(vals, lbls)) > 4
-            
+
             if (!is.null(type)) {
                 # at least two non missing values are needed to calculate sd()
                 printnum <- printnum | (length(vals) > 2 & grepl("num", type))
             }
-            
+
             if (printnum) { # numeric variable
                 cat(paste(
                     s3,
@@ -504,7 +715,7 @@
                     enter,
                     sep = ""
                 ))
-                
+
                 cat(paste(
                     s3,
                     "<", ns, "sumStat type=\"mean\">",
@@ -516,7 +727,7 @@
                     enter,
                     sep = ""
                 ))
-                
+
                 cat(paste(
                     s3,
                     "<", ns, "sumStat type=\"medn\">",
@@ -528,7 +739,7 @@
                     enter,
                     sep = ""
                 ))
-                
+
                 cat(paste(
                     s3,
                     "<", ns, "sumStat type=\"stdev\">",
@@ -540,16 +751,16 @@
                     enter,
                     sep = ""
                 ))
-            
+
             }
         }
-        
+
         if (!is.null(lbls)) {
 
             tbl <- table(data[[names(dataDscr)[i]]]) # what is the difference from data[[i]] ?
-            
+
             for (v in seq(length(lbls))) {
-                
+
                 ismiss <- is.element(lbls[v], na_values)
                 if (length(na_range) > 0) {
                     ismiss <- ismiss | (lbls[v] >= na_range[1] & lbls[v] <= na_range[2])
@@ -570,7 +781,7 @@
                     enter,
                     sep = ""
                 ))
-                
+
                 cat(paste(
                     s4,
                     "<", ns, "labl", xmlang, ">",
@@ -579,17 +790,17 @@
                     enter,
                     sep = ""
                 ))
-                
+
                 if (!is.null(data)) {
                     freq <- tbl[match(lbls[v], names(tbl))]
                     cat(paste(
                         s4,
-                        "<", ns, "catStat type=\"freq\">", 
+                        "<", ns, "catStat type=\"freq\">",
                         ifelse(
                             is.na(freq),
                             0,
                             format(freq, scientific = FALSE)
-                        ), 
+                        ),
                         "</", ns, "catStat>",
                         enter,
                         sep = ""
@@ -630,12 +841,12 @@
             ))
             cat(paste(s3, "</", ns, "txt>", enter, sep = ""))
         }
-        
+
         cat(paste(s2, "</", ns, "var>", enter, sep = ""))
     }
-    
+
     cat(paste(s1, "</", ns, "dataDscr>", enter, sep = ""))
     cat(paste(s1, "<", ns, "otherMat level=\"", level, "\"></", ns, "otherMat>", enter, sep = ""))
     cat(paste(s0, "</", ns, "codeBook>", enter, sep = ""))
-    
+
 }
