@@ -23,24 +23,101 @@
 # (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 # SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
-`getMetadata` <- 
-function(x, save = FALSE, declared = TRUE, OS = "Windows", encoding = "UTF-8", ...) {
-    
+#' @name getMetadata
+#'
+#' @title Extract metadata information
+#'
+#' @description
+#' Extract a list containing the variable labels, value labels and any
+#' available information about missing values.
+#'
+#' @details
+#' This function reads an XML file containing a DDI codebook version 2.5, or an
+#' SPSS or Stata file and returns a list containing the variable labels, value
+#' labels, plus some other useful information.
+#'
+#' It additionally attempts to automatically detect a type for each variable:
+#' \tabular{rl}{
+#'   \bold{\code{cat}}: \tab categorical variable using numeric values\cr
+#'   \bold{\code{catchar}}: \tab categorical variable using character values\cr
+#'   \bold{\code{catnum}}: \tab categorical variable for which numerical summaries\cr
+#'   \tab can be calculated (ex. a 0...10 Likert response scale)\cr
+#'   \bold{\code{num}}: \tab numerical\cr
+#'   \bold{\code{numcat}}: \tab numerical variable with few enough values (ex. number of children)\cr
+#'   \tab for which a table of frequencies is possible in addition to frequencies
+#' }
+#'
+#' By default, this function extracts the metadata into an R list object, but when
+#' the argument \code{save} is activated, the argument \code{OS} (case insensitive)
+#' can be either:\cr
+#' \code{"Windows"} (default), or \code{"Win"},\cr
+#' \code{"MacOS"}, \code{"Darwin"}, \code{"Apple"}, \code{"Mac"},\cr
+#' \code{"Linux"}.\cr
+#'
+#' The end of line separator changes only when the target OS is different from the
+#' running OS.
+#'
+#' For the moment, only DDI version 2.5 (Codebook) is supported, but DDI Lifecycle
+#' is planned to be implemented.
+#'
+#' @examples
+#' x <- data.frame(
+#'     A = declared(
+#'         c(1:5, -92),
+#'         labels = c(Good = 1, Bad = 5, NR = -92),
+#'         na_values = -92
+#'     ),
+#'     C = declared(
+#'         c(1, -91, 3:5, -92),
+#'         labels = c(DK = -91, NR = -92),
+#'         na_values = c(-91, -92)
+#'     )
+#' )
+#'
+#' getMetadata(x)$dataDscr
+#'
+#' @return
+#' An R list roughly equivalent to a DDI Codebook, containing all variables,
+#' their corresponding variable labels and value labels, and (if applicable)
+#' missing values if imported and found.
+#'
+#' @author Adrian Dusa
+#'
+#' @param x A path to a file, or a data frame object
+#'
+#' @param save Logical, save an .R file in the same directory
+#'
+#' @param declared Logical, embed the data as a declared object
+#'
+#' @param OS The target operating system, for the eol - end of line separator,
+#' if saving the file
+#'
+#' @param encoding The character encoding used to read a file
+#'
+#' @param ... .Additional arguments for this function (internal uses only)
+#'
+#' @export
+
+`getMetadata` <- function(
+    x, save = FALSE, declared = TRUE, OS = "Windows", encoding = "UTF-8", ...
+) {
+
     # TODO: detect DDI version or ask the version through a dedicated argument
     # http://www.ddialliance.org/Specification/DDI-Codebook/2.5/XMLSchema/field_level_documentation.html
-    
+
     dots <- list(...)
+    stdyDscr <- NULL
 
     user_na <- TRUE # force reading the value labels
     if (
-        is.element("user_na", names(dots)) && is.atomic(dots$user_na) && 
+        is.element("user_na", names(dots)) && is.atomic(dots$user_na) &&
         length(dots$user_na) == 1 && is.logical(dots$user_na)
     ) {
         user_na <- dots$user_na
     }
-    
+
     embed <- isTRUE(dots$embed)
-    
+
     if (is.data.frame(x)) {
         error <- TRUE
         i <- 1
@@ -70,16 +147,21 @@ function(x, save = FALSE, declared = TRUE, OS = "Windows", encoding = "UTF-8", .
             return(codeBook)
         }
     }
-    
+    else {
+        if (!is.atomic(x) || !is.character(x) || length(x) != 1) {
+            admisc::stopError("A path should be a string of length 1")
+        }
+    }
+
     enter <- getEnter(OS)
-    
+
     fromsetupfile <- isTRUE(dots$fromsetupfile)
-    
+
     tp <- treatPath(x, type = "*")
-    
+
     singlefile <- length(tp$files) == 1
     notes <- NA
-    
+
     if (!fromsetupfile & !singlefile) {
         cat("Processing:\n")
     }
@@ -87,21 +169,27 @@ function(x, save = FALSE, declared = TRUE, OS = "Windows", encoding = "UTF-8", .
     data <- NULL
 
     result <- vector(mode = "list", length = length(tp$files))
-    
+
     for (ff in seq(length(result))) {
         if (!fromsetupfile & !singlefile) {
             cat(tp$files[ff], "\n")
         }
-        
+
         if (tp$fileext[ff] == "XML") {
 
             codeBook <- list()
             xml <- getXML(file.path(tp$completePath, tp$files[ff]))
-            
+
+            children <- xml2::xml_children(xml)
+            nms <- xml2::xml_name(children)
+            if (is.element("stdyDscr", nms)) {
+                stdyDscr <- xml2::as_list(children[[which(nms == "stdyDscr")]])
+            }
+
             # lapply(xml_find_all(xml, "/d1:codeBook/d1:dataDscr/d1:var"), function(x) {
             #     list(label = admisc::trimstr(xml_text(xml_find_first(x, "d1:labl"))))
             # })
-            
+
             xmlns <- xml2::xml_ns(xml)
             # d1  <-> ddi:codebook:2_5"
             # xsi <-> http://www.w3.org/2001/XMLSchema-instance
@@ -117,7 +205,7 @@ function(x, save = FALSE, declared = TRUE, OS = "Windows", encoding = "UTF-8", .
                 codeBook$xmlns <- dns
             }
             dns <- paste0(dns, ":")
-            
+
             ### Unfortunately this does not work because some variables don't always have labels
             ### and we'll end up having a vector of labels that is shorter than the number of variables
             # xpath <- sprintf("/%scodeBook/%sdataDscr/%svar/%slabl", dns, dns, dns, dns)
@@ -132,13 +220,13 @@ function(x, save = FALSE, declared = TRUE, OS = "Windows", encoding = "UTF-8", .
             notes <- xml2::xml_text(xml2::xml_find_first(xml, xpath))
 
             codeBook$dataDscr <- lapply(varlab, function(x) list(label = x))
-            
+
             xpath <- sprintf("/%scodeBook/%sdataDscr/%svar/@name", dns, dns, dns)
             names(codeBook$dataDscr) <- admisc::trimstr(xml2::xml_text(xml2::xml_find_all(xml, xpath)))
-            
+
             for (i in seq(length(codeBook$dataDscr))) {
                 # nms <- xml_name(xml_contents(xml_find_all(xml, sprintf("/d1:codeBook/d1:dataDscr/d1:var[%s]", i))))
-                
+
                 # xpath <- sprintf("/%scodeBook/%sdataDscr/%svar[%s]", dns, dns, dns, i)
                 # vars_i <- xml2::xml_find_first(xml, xpath)
 
@@ -155,7 +243,7 @@ function(x, save = FALSE, declared = TRUE, OS = "Windows", encoding = "UTF-8", .
                     if (is.na(na_range[1])) na_range[1] <- -Inf
                     if (is.na(na_range[2])) na_range[2] <- Inf
                 }
-                
+
                 xpath <- sprintf("%scatgry/%scatValu", dns, dns)
                 values <- cleanup(xml2::xml_text(xml2::xml_find_all(vars[i], xpath)))
 
@@ -163,32 +251,32 @@ function(x, save = FALSE, declared = TRUE, OS = "Windows", encoding = "UTF-8", .
                 vformat <- xml2::xml_find_first(vars[i], xpath)
                 type <- xml2::xml_attr(vformat, "type")
                 varFormat <- xml2::xml_text(vformat)
-                
+
                 if (length(values) > 0) {
-                    
+
                     catgry <- xml2::xml_find_all(vars[i], sprintf("%scatgry", dns))
-                    
+
                     na_values <- c(na_values, values[unlist(lapply(catgry, function(x) {
                         grepl("Y", xml2::xml_attr(x, "missing"))
                     }))])
-                    
+
                     labl <- unlist(lapply(catgry, function(x) {
                         xml2::xml_text(xml2::xml_find_first(x, sprintf("%slabl", dns)))
                     }))
-                    
+
                     values <- values[!is.na(labl)]
                     labl <- cleanup(labl[!is.na(labl)])
-                    
+
                     if (admisc::possibleNumeric(values)) {
                         values <- admisc::asNumeric(values)
                     }
-                    
+
                     codeBook$dataDscr[[i]][["labels"]] <- values
                     names(codeBook$dataDscr[[i]][["labels"]]) <- labl
                 }
-                
+
                 if (length(na_values) > 0) {
-                    
+
                     if (admisc::possibleNumeric(na_values) & admisc::possibleNumeric(values)) {
                         na_values <- admisc::asNumeric(na_values)
                     }
@@ -198,7 +286,7 @@ function(x, save = FALSE, declared = TRUE, OS = "Windows", encoding = "UTF-8", .
                     if (!is.null(na_range) && is.numeric(na_values)) {
                         na_values <- na_values[na_values < na_range[1] | na_values > na_range[2]]
                     }
-                    
+
                     if (length(na_values) > 0) {
                         codeBook$dataDscr[[i]]$na_values <- na_values
                     }
@@ -280,11 +368,11 @@ function(x, save = FALSE, declared = TRUE, OS = "Windows", encoding = "UTF-8", .
             # else if (tp$fileext[ff] == "SAS7BDAT") {
             #     data <- haven::read_sas(file.path(tp$completePath, tp$files[ff]))
             # }
-            
+
             codeBook <- list()
             codeBook$dataDscr <- collectMetadata(data)
         }
-        
+
         codeBook$fileDscr$fileName <- tp$files[ff]
 
         filetypes <- c("SPSS", "SPSS", "Stata", "SAS", "R", "DDI", "Excel", "Excel")
@@ -293,14 +381,14 @@ function(x, save = FALSE, declared = TRUE, OS = "Windows", encoding = "UTF-8", .
         codeBook$fileDscr$fileType <- filetypes[which(fileexts == tp$fileext[ff])]
 
         result[[ff]] <- codeBook
-        
+
         if (save) {
-            
+
             indent <- 4
             if (is.element("indent", names(dots))) {
                 indent <- dots$indent
             }
-            
+
             writeRlist(
                 codeBook$dataDscr,
                 OS = OS,
@@ -308,12 +396,12 @@ function(x, save = FALSE, declared = TRUE, OS = "Windows", encoding = "UTF-8", .
                 dirpath = tp$completePath,
                 filename = tp$filenames[ff]
             )
-            
+
         }
     }
 
     names(result) <- tp$filenames
-    
+
     if (singlefile) {
         if (!is.na(notes)) {
             if (grepl("# start data #", notes)) {
@@ -340,11 +428,15 @@ function(x, save = FALSE, declared = TRUE, OS = "Windows", encoding = "UTF-8", .
                 embed <- TRUE
             }
         }
-        
+
         if (embed & !is.null(data)) {
             codeBook$fileDscr$datafile <- data
         }
-        
+
+        if (!is.null(stdyDscr)) {
+            codeBook$stdyDscr <- stdyDscr
+        }
+
         return(codeBook)
     }
     else {
